@@ -15,53 +15,29 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#	include "config.h"
+#include "config.h"
 #endif
 
-/* External libs */
-#include <bson.h>
-#include <mongoc.h>
-
-/* PHP Core stuff */
 #include <php.h>
-#include <php_ini.h>
-#include <ext/standard/info.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/spl/spl_iterators.h>
-/* Our Compatability header */
+
 #include "phongo_compat.h"
-
-/* Our stuffz */
 #include "php_phongo.h"
-#include "php_bson.h"
-#include <ext/spl/spl_exceptions.h>
 
+zend_class_entry* php_phongo_exception_ce;
 
-PHONGO_API zend_class_entry *php_phongo_exception_ce;
-
-/* {{{ MongoDB\Driver\Exception */
-
+/* {{{ MongoDB\Driver\Exception\Exception function entries */
 static zend_function_entry php_phongo_exception_me[] = {
 	PHP_FE_END
 };
-
 /* }}} */
 
-
-/* {{{ PHP_MINIT_FUNCTION */
-PHP_MINIT_FUNCTION(Exception)
+void php_phongo_exception_init_ce(INIT_FUNC_ARGS) /* {{{ */
 {
 	zend_class_entry ce;
-	(void)type;(void)module_number;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver\\Exception", "Exception", php_phongo_exception_me);
 	php_phongo_exception_ce = zend_register_internal_interface(&ce TSRMLS_CC);
-
-	return SUCCESS;
-}
-/* }}} */
-
-
+} /* }}} */
 
 /*
  * Local variables:
